@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2015 David R. MacIver
+# Most of this work is copyright (C) 2013-2016 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -58,7 +58,7 @@ def default_new_style_executor(data, function):
     return function(data)
 
 
-class TestRunner(object):
+class ConjectureRunner(object):
 
     def hypothesis_execute_example_with_data(self, data, function):
         return function(data)
@@ -67,7 +67,7 @@ class TestRunner(object):
 def new_style_executor(runner):
     if runner is None:
         return default_new_style_executor
-    if isinstance(runner, TestRunner):
+    if isinstance(runner, ConjectureRunner):
         return runner.hypothesis_execute_example_with_data
 
     old_school = executor(runner)
