@@ -1,12 +1,13 @@
 # coding=utf-8
 #
-# This file is part of Hypothesis (https://github.com/DRMacIver/hypothesis)
+# This file is part of Hypothesis, which may be found at
+# https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2015 David R. MacIver
+# Most of this work is copyright (C) 2013-2016 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
-# https://github.com/DRMacIver/hypothesis/blob/master/CONTRIBUTING.rst for a
-# full list of people who may hold copyright, and consult the git log if you
-# need to determine who owns an individual contribution.
+# CONTRIBUTING.rst for a full list of people who may hold copyright, and
+# consult the git log if you need to determine who owns an individual
+# contribution.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
@@ -25,7 +26,10 @@ import math
 import codecs
 import platform
 import importlib
+from gzip import GzipFile
+from base64 import b64decode, b64encode
 from decimal import Context, Decimal, Inexact
+from hashlib import sha1
 from collections import namedtuple
 
 try:
@@ -37,45 +41,38 @@ except ImportError:  # pragma: no cover
 
 PY2 = sys.version_info[0] == 2
 PY3 = sys.version_info[0] == 3
-PYPY = platform.python_implementation() == u'PyPy'
-PY26 = sys.version_info[:2] == (2, 6)
+PYPY = platform.python_implementation() == 'PyPy'
 NO_ARGSPEC = sys.version_info[:2] >= (3, 5)
 HAS_SIGNATURE = sys.version_info[:2] >= (3, 3)
+CAN_UNPACK_BYTE_ARRAY = sys.version_info[:3] >= (2, 7, 4)
 
 WINDOWS = platform.system() == 'Windows'
 
-if PY26:
-    _special_floats = {
-        float(u'inf'): Decimal(u'Infinity'),
-        float(u'-inf'): Decimal(u'-Infinity'),
-    }
+if sys.version_info[:2] <= (2, 6):
+    raise ImportError(
+        'Hypothesis is not supported on Python versions before 2.7'
+    )
 
-    def float_to_decimal(f):
-        """Convert a floating point number to a Decimal with no loss of
-        information."""
-        if f in _special_floats:
-            return _special_floats[f]
-        elif math.isnan(f):
-            return Decimal(u'NaN')
-        n, d = f.as_integer_ratio()
-        numerator, denominator = Decimal(n), Decimal(d)
-        ctx = Context(prec=60)
-        result = ctx.divide(numerator, denominator)
-        while ctx.flags[Inexact]:
-            ctx.flags[Inexact] = False
-            ctx.prec *= 2
-            result = ctx.divide(numerator, denominator)
-        return result
-else:
-    def float_to_decimal(f):
-        return Decimal(f)
+
+def bit_length(n):
+    return n.bit_length()
+
+
+def float_to_decimal(f):
+    return Decimal(f)
 
 
 if PY3:
+    def str_to_bytes(s):
+        return s.encode(a_good_encoding())
+
+    def int_to_text(i):
+        return str(i)
+
     text_type = str
     binary_type = bytes
     hrange = range
-    ARG_NAME_ATTRIBUTE = u'arg'
+    ARG_NAME_ATTRIBUTE = 'arg'
     integer_types = (int,)
     hunichr = chr
     from functools import reduce
@@ -89,11 +86,78 @@ if PY3:
     def escape_unicode_characters(s):
         return codecs.encode(s, 'unicode_escape').decode('ascii')
 
+    def print_unicode(x):
+        print(x)
+
     exec("""
 def quiet_raise(exc):
     raise exc from None
 """)
+
+    def int_from_bytes(data):
+        return int.from_bytes(data, 'big')
+
+    def int_to_bytes(i, size):
+        return i.to_bytes(size, 'big')
+
+    def bytes_from_list(ls):
+        return bytes(ls)
+
+    def to_bytes_sequence(ls):
+        return bytes(ls)
+
+    def zero_byte_sequence(n):
+        return bytes(n)
+
+    from time import monotonic as benchmark_time
 else:
+    import struct
+
+    def zero_byte_sequence(n):
+        return b'\0' * n
+
+    def int_from_bytes(data):
+        assert isinstance(data, bytearray)
+        if CAN_UNPACK_BYTE_ARRAY:
+            unpackable_data = data
+        else:
+            unpackable_data = bytes(data)
+        result = 0
+        i = 0
+        while i + 4 <= len(data):
+            result <<= 32
+            result |= struct.unpack('>I', unpackable_data[i:i + 4])[0]
+            i += 4
+        while i < len(data):
+            result <<= 8
+            result |= data[i]
+            i += 1
+        return int(result)
+
+    def int_to_bytes(i, size):
+        assert i >= 0
+        result = bytearray(size)
+        j = size - 1
+        while i and j >= 0:
+            result[j] = i & 255
+            i >>= 8
+            j -= 1
+        if i:
+            raise OverflowError('int too big to convert')
+        return hbytes(result)
+
+    def bytes_from_list(ls):
+        return bytes(bytearray(ls))
+
+    def to_bytes_sequence(ls):
+        return bytearray(ls)
+
+    def str_to_bytes(s):
+        return s
+
+    def int_to_text(i):
+        return str(i).decode('ascii')
+
     VALID_PYTHON_IDENTIFIER = re.compile(
         r"^[a-zA-Z_][a-zA-Z0-9_]*$"
     )
@@ -102,22 +166,9 @@ else:
         return VALID_PYTHON_IDENTIFIER.match(s)
 
     def unicode_safe_repr(x):
-        try:
-            r = repr(x)
-        except UnicodeEncodeError:
-            r = type(x).__repr__(x)
-            from hypothesis.settings import note_deprecation
-            note_deprecation((
-                'Type %s has a broken repr implementation. Calling __repr__ '
-                'on it returned %r, which cannot be represented as ASCII '
-                'text. This is not permitted in Python 2. Hypothesis is '
-                'currently working around this, but will stop doing so in '
-                'Hypothesis 2.0. You should fix your code.'
-            ) % (type(x).__name__, r))
-        if isinstance(r, unicode):
-            return r
-        else:
-            return r.decode(a_good_encoding())
+        r = repr(x)
+        assert isinstance(r, str)
+        return r.decode(a_good_encoding())
 
     text_type = unicode
     binary_type = str
@@ -156,7 +207,7 @@ else:
                         i += step
             return shimrange()
 
-    ARG_NAME_ATTRIBUTE = u'id'
+    ARG_NAME_ATTRIBUTE = 'id'
     integer_types = (int, long)
     hunichr = unichr
     reduce = reduce
@@ -164,16 +215,19 @@ else:
     def escape_unicode_characters(s):
         return codecs.encode(s, 'string_escape')
 
+    def print_unicode(x):
+        if isinstance(x, unicode):
+            x = x.encode(a_good_encoding())
+        print(x)
+
     def quiet_raise(exc):
         raise exc
 
+    from time import time as benchmark_time
+
 
 def a_good_encoding():
-    result = sys.getdefaultencoding()
-    if result == u'ascii':
-        return u'utf-8'
-    else:
-        return result
+    return 'utf-8'
 
 
 def to_unicode(x):
@@ -189,7 +243,7 @@ def qualname(f):
     except AttributeError:
         pass
     try:
-        return f.im_class.__name__ + u'.' + f.__name__
+        return f.im_class.__name__ + '.' + f.__name__
     except AttributeError:
         return f.__name__
 
@@ -233,14 +287,18 @@ def signature_argspec(f):
 
 
 if NO_ARGSPEC:
-    getargspec = signature_argspec
     ArgSpec = FakeArgSpec
 else:
-    from inspect import getargspec, ArgSpec
+    from inspect import ArgSpec
+
+if PY2:
+    from inspect import getargspec
+else:
+    getargspec = signature_argspec
 
 
 importlib_invalidate_caches = getattr(
-    importlib, u'invalidate_caches', lambda: ())
+    importlib, 'invalidate_caches', lambda: ())
 
 
 if PY2:
@@ -297,3 +355,121 @@ def update_code_location(code, newfile, newlineno):
     unpacked[CODE_FIELD_ORDER.index('co_filename')] = newfile
     unpacked[CODE_FIELD_ORDER.index('co_firstlineno')] = newlineno
     return type(code)(*unpacked)
+
+
+class compatbytes(bytearray):
+    __name__ = 'bytes'
+
+    def __init__(self, *args, **kwargs):
+        bytearray.__init__(self, *args, **kwargs)
+        self.__hash = None
+
+    def __str__(self):
+        return bytearray.__str__(self)
+
+    def __repr__(self):
+        return 'compatbytes(b%r)' % (str(self),)
+
+    def __hash__(self):
+        if self.__hash is None:
+            self.__hash = hash(str(self))
+        return self.__hash
+
+    def count(self, value):
+        c = 0
+        for w in self:
+            if w == value:
+                c += 1
+        return c
+
+    def index(self, value):
+        for i, v in enumerate(self):
+            if v == value:
+                return i
+        raise ValueError('Value %r not in sequence %r' % (value, self))
+
+    def __add__(self, value):
+        return compatbytes(bytearray.__add__(self, value))
+
+    def __radd__(self, value):
+        return compatbytes(bytearray.__add__(value, self))
+
+    def __mul__(self, value):
+        return compatbytes(bytearray.__mul__(self, value))
+
+    def __rmul__(self, value):
+        return compatbytes(bytearray.__rmul__(self, value))
+
+    def __getitem__(self, *args, **kwargs):
+        r = bytearray.__getitem__(self, *args, **kwargs)
+        if isinstance(r, bytearray):
+            return compatbytes(r)
+        else:
+            return r
+
+    __setitem__ = None
+
+    def join(self, parts):
+        result = bytearray()
+        first = True
+        for p in parts:
+            if not first:
+                result.extend(self)
+            first = False
+            result.extend(p)
+        return compatbytes(result)
+
+    def __contains__(self, value):
+        return any(v == value for v in self)
+
+if PY2:
+    hbytes = compatbytes
+    reasonable_byte_type = bytearray
+    string_types = (str, unicode)
+else:
+    hbytes = bytes
+    reasonable_byte_type = bytes
+    string_types = (str,)
+
+
+if PY2:
+    def to_str(s):
+        if isinstance(s, unicode):
+            return s.encode(a_good_encoding())
+        assert isinstance(s, str)
+        return s
+else:
+    def to_str(s):
+        return s
+
+
+def cast_unicode(s, encoding=None):
+    if isinstance(s, bytes):
+        return s.decode(encoding or a_good_encoding(), 'replace')
+    return s
+
+
+def get_stream_enc(stream, default=None):
+    return getattr(stream, 'encoding', None) or default
+
+if PY3:
+    FileNotFoundError = FileNotFoundError
+else:
+    FileNotFoundError = IOError
+
+# We need to know what sort of exception gets thrown when you try to write over
+# an existing file where you're not allowed to. This is rather less consistent
+# between versions than might be hoped.
+if PY3:
+    # Note: This only works on >= 3.3, but we only support >= 3.3 so that's
+    # fine
+    FileExistsError = FileExistsError
+
+elif WINDOWS:
+    FileExistsError = WindowsError
+
+else:
+    # This doesn't happen in this case: We're not on windows and don't support
+    # the x flag because it's Python 2, so there are no places where this can
+    # be thrown.
+    FileExistsError = None
